# Generated by Django 2.1.7 on 2019-02-14 07:27

import uuid

import phonenumber_field.modelfields
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="SMSVerification",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("modified_at", models.DateTimeField(auto_now=True)),
                ("otp", models.CharField(max_length=120, verbose_name="OTP")),
                (
                    "phone_number",
                    phonenumber_field.modelfields.PhoneNumberField(
                        max_length=128, verbose_name="Phone Number"
                    ),
                ),
                (
                    "session_code",
                    models.CharField(
                        max_length=500, verbose_name="Device Session Code"
                    ),
                ),
            ],
            options={
                "verbose_name": "SMS Verification",
                "verbose_name_plural": "SMS Verifications",
                "db_table": "sms_verification",
                "ordering": ("-modified_at",),
            },
        ),
        migrations.AlterUniqueTogether(
            name="smsverification",
            unique_together={("otp", "phone_number", "session_code")},
        ),
    ]
